# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('custom_field', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='customfield',
            name='field_choices',
            field=models.CharField(help_text=b'List the choices you want displayed, seperated by commas. This is only valid for Dropdown, Multiple, and Checkbox field types', max_length=2000, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='customfield',
            name='is_required',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='customfield',
            name='default_value',
            field=models.CharField(help_text=b'You may leave blank. For Boolean use True or False', max_length=5000, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='customfield',
            name='field_type',
            field=models.CharField(default=b't', max_length=1, choices=[(b't', b'Text'), (b'a', b'Large Text Field'), (b'i', b'Integer'), (b'f', b'Floating point decimal'), (b'b', b'Boolean (Yes/No)'), (b'm', b'Dropdown Choices'), (b'd', b'Date')]),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='customfield',
            name='name',
            field=models.CharField(max_length=150),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='customfieldvalue',
            name='value',
            field=models.CharField(max_length=5000, null=True, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomField',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=75)),
                ('field_type', models.CharField(default=b't', max_length=1, choices=[(b't', b'Text'), (b'i', b'Integer'), (b'b', b'Boolean (Yes/No)')])),
                ('default_value', models.CharField(help_text=b'You may leave blank. For Boolean use True or False', max_length=255, blank=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='CustomFieldValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.CharField(max_length=255, null=True, blank=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True)),
                ('field', models.ForeignKey(related_name='instance', to='custom_field.CustomField')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='customfieldvalue',
            unique_together=set([('field', 'object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='customfield',
            unique_together=set([('name', 'content_type')]),
        ),
    ]



from django.contrib import admin

from .models import CustomField
from .custom_field import CustomFieldAdmin


class CFAdmin(CustomFieldAdmin):
    list_display = ('content_type', 'name')
    list_filter = ('content_type',)
    search_fields = ('content_type__name', 'name')
admin.site.register(CustomField, CFAdmin)



from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.test import TestCase
from .models import CustomField, CustomFieldValue
from django.core import urlresolvers


class CustomFieldTest(TestCase):
    def setUp(self):
        custom_field_ct = ContentType.objects.get(app_label="custom_field",
                                                  model="customfield")
        self.custom_field = CustomField.objects.create(
            name="test_field",
            content_type=custom_field_ct,
            field_type="i",
        )
        self.user_custom_field = CustomField.objects.create(
            name="test_user_field",
            content_type=custom_field_ct,
            field_type="i",
            default_value=42,
        )
        user = User.objects.create_user('temporary', 'temporary@gmail.com',
                                        'temporary')
        user.is_staff = True
        user.is_superuser = True
        user.save()
        self.client.login(username='temporary', password='temporary')

    def test_validation(self):
        custom_value = CustomFieldValue.objects.create(
            field=self.custom_field,
            value='5',
            object_id=self.custom_field.id,
        )
        custom_value.clean()
        custom_value.save()
        self.assertEquals(custom_value.value, '5')
        custom_value.value = 'fdsf'
        try:
            custom_value.clean()
            self.fail('Was able to save string as custom integer field!')
        except ValidationError:
            pass

    def test_admin(self):
        change_url = urlresolvers.reverse(
            'admin:custom_field_customfield_change', args=[1])
        response = self.client.get(change_url)
        self.assertContains(response, '42')
        response = self.client.get(change_url)
        # Make sure we aren't adding it on each get
        self.assertContains(response, '42')



from django import forms
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.contrib.contenttypes import fields
from django.utils.encoding import python_2_unicode_compatible
import sys


if sys.version < '3':
    text_type = unicode
else:
    text_type = str


@python_2_unicode_compatible
class CustomField(models.Model):
    """
    A field abstract -- it describe what the field is.  There are one of these
    for each custom field the user configures.
    """
    name = models.CharField(max_length=150)
    content_type = models.ForeignKey(ContentType)
    field_type = models.CharField(
        max_length=1,
        choices=(
            ('t', 'Text'),
            ('a', 'Large Text Field'),
            ('i', 'Integer'),
            ('f', 'Floating point decimal'),
            ('b', 'Boolean (Yes/No)'),
            ('m', 'Dropdown Choices'),
            ('d', 'Date'),
        ),
        default='t')
    default_value = models.CharField(
        max_length=5000,
        blank=True,
        help_text="You may leave blank. For Boolean use True or False")
    is_required = models.BooleanField(default=False)
    field_choices = models.CharField(
        max_length=2000,
        blank=True,
        help_text="List the choices you want displayed, seperated by commas. "
        "This is only valid for Dropdown, Multiple, and Checkbox field types",
    )

    def get_value_for_object(self, obj):
        return CustomFieldValue.objects.get_or_create(
            field=self,
            object_id=obj.id)[0]

    def __str__(self):
        return self.name

    def get_form_field(self):
        universal_kwargs = {
            'initial': self.default_value,
            'required': self.is_required,
        }
        if self.field_type == "b":
            check_choices = (('True', 'True'), ('False', 'False'))
            return forms.ChoiceField(choices=check_choices, **universal_kwargs)
        elif self.field_type == "i":
            return forms.IntegerField(**universal_kwargs)
        elif self.field_type == "f":
            return forms.FloatField(**universal_kwargs)
        elif self.field_type == "a":
            return forms.CharField(widget=forms.Textarea, **universal_kwargs)
        elif self.field_type == "m":
            choices = self.field_choices.split(',')
            if self.is_required is True:
                select_choices = ()
            else:
                select_choices = (('', '---------'),)
            for choice in choices:
                select_choices = select_choices + ((choice, choice),)
            return forms.ChoiceField(
                choices=select_choices, **universal_kwargs)
        elif self.field_type == "d":
            return forms.DateField(**universal_kwargs)
        return forms.CharField(**universal_kwargs)

    class Meta:
        unique_together = ('name', 'content_type')


@python_2_unicode_compatible
class CustomFieldValue(models.Model):
    """
    A field instance -- contains the actual data.  There are many of these, for
    each value that corresponds to a CustomField for a given model.
    """
    field = models.ForeignKey(CustomField, related_name='instance')
    value = models.CharField(max_length=5000, blank=True, null=True)
    object_id = models.PositiveIntegerField()
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    content_object = fields.GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return text_type(self.value)

    def save(self, *args, **kwargs):
        super(CustomFieldValue, self).save(*args, **kwargs)
        if not self.content_type:
            self.content_type = self.field.content_type
            self.save()

    def clean(self):
        form_field = self.get_form_field()
        form_field.clean(self.value)
        return super(CustomFieldValue, self).clean()

    def get_form_field(self):
        return self.field.get_form_field()

    class Meta:
        unique_together = ('field', 'object_id')
